/*
 * http://www.wayforward.net/avGPS/libav/av_dir_win.c
 * Copyright(c) 2005, 2008 Wayforward LLC
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
#include "av_dir.h"
#include "av_err.h"

static const tchar_t FINDFIRSTFILE_FN[] = T("FindFirstFile");
static const tchar_t FINDNEXTFILE_FN[] = T("FindNextFile");

const tchar_t *
dir_first(const tchar_t *fn, struct directory *pd) {
	static const tchar_t SEARCH_FMT[] = T("%s\\*.*");

	tchar_t search[1025];
	const tchar_t *result;

	wsprintf(search, SEARCH_FMT, fn);

	pd->hDir = FindFirstFile(search, &pd->wfd);

	if (pd->hDir == INVALID_HANDLE_VALUE) {
		err_push_last_error(FINDFIRSTFILE_FN, search);
		result = NULL;
	}
	else if ((pd->wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		 != FILE_ATTRIBUTE_DIRECTORY &&
		 pd->wfd.cFileName[0] != T('.'))
		result = pd->wfd.cFileName;
	else {
		result = dir_next(pd);

		if (result == NULL) {
			err_push_last_error(FINDNEXTFILE_FN, search);
			(void) FindClose(pd->hDir);
		}
	}

	return result;
}

const tchar_t *
dir_next(struct directory *pd) {
	while (FindNextFile(pd->hDir, &pd->wfd)) {
		if ((pd->wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			!= FILE_ATTRIBUTE_DIRECTORY &&
		    pd->wfd.cFileName[0] != T('.'))
		return pd->wfd.cFileName;
	}

	return NULL;
}

void
dir_close(struct directory *pd) {
	(void) FindClose(pd->hDir);
}

/*
 * Overrides for Emacs so that we follow Linux's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the
 * end of the file.
 * ----------------------------------------------------------------------
 * Local Variables:
 * c-file-style: "linux"
 * End:
 */
