/*
 * http://www.wayforward.net/avGPS/include/av_err.h
 * Copyright(c) 2005, 2008 Wayforward LLC
 *
 * Stack-based error handling.  Functions that detect errors "push"
 * error information on a stack.  Calling functions can "pop" the
 * information and display it in a suitable fashion.
 * ---
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
#ifndef AV_ERR_H
#define AV_ERR_H

#include "av_str.h"

#ifdef __cplusplus
extern "C" {
#endif

	void
	err_init(void);

	void
	err_push(const tchar_t *function, const tchar_t *object,
		 const tchar_t *reason);

	/**
	 * Describe the location of an error if the error occurred
	 * parsing a text file.
	 */
	void
	err_push_source(const tchar_t *filename, unsigned long lineno,
	                const tchar_t *reason);

# ifdef _WIN32
	/**
	 * Push Windows API GetLastError() onto the error stack.
	 * The _function_ argument names the RTL call that failed.
	 *
	 * Identical to err_push_win32_error(function, GetLastError());
	 */
	void
	err_push_last_error(const tchar_t *function, const tchar_t *object);

	void
	err_push_win32_error(const tchar_t *function, const tchar_t *object,
			     unsigned long err);

# else

	/**
	 * Push C RTL 'errno' onto the error stack.  The _function_
	 * argument names the system call that failed.
	 *
	 * Identical to err_push_c_error(function, object, errno);
	 */
	void
	err_push_errno(const tchar_t *function, const tchar_t *object);

	/**
	 * Pushes a C RTL-style error (errno) onto the error
	 * stack.
	 */
	void
	err_push_c_error(const tchar_t *function, const tchar_t *object, int err);

#endif

	void
	err_exit(const tchar_t *desc);

	const tchar_t *
	err_text(void);

	void
	err_clear(void);

#ifdef __cplusplus
}
#endif

#endif /* AV_ERR_H */

/*
 * Overrides for Emacs so that we follow Linux's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the
 * end of the file.
 * ----------------------------------------------------------------------
 * Local Variables:
 * c-file-style: "linux"
 * End:
 */
