/*
 * http://www.wayforward.net/avGPS/include/av_map.h
 * Copyright(c) 2005, 2008 Wayforward LLC
 *
 * There are two different versions of this interface:
 * hiresmap.c:  reads .arr.gz files directly, and provides a
 * 450x450 tile that takes up 1/8 degree; and
 * loresmap.c:  reads a compiled .dat file, provides a
 * 300x300 tile that takes up 1/4 degree.
 *
 * To use hiresmap, define HIRESMAP and include this header.  Otherwise
 * the loresmap will be used by default
 * --------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
#ifndef AV_MAP_H
#define AV_MAP_H

#include "av_str.h"

#ifdef HIRESMAP
# define TILE_SIZE (450)
# define TILES_PER_DEGREE (8)
# define index_north  index_north_450x8
# define index_south  index_south_450x8
# define index_east   index_east_450x8
# define index_west   index_west_450x8
# define pack_index   pack_index_450x8
# define unpack_index unpack_index_450x8
# define latlon2index latlon2index_450x8
# define index2latlon index2latlon_450x8
# define index_filename index_filename_450x8
# define load_tile    load_tile_450x8
# define fetch_tile   fetch_tile_450x8

#else
# define TILE_SIZE (300)
# define TILES_PER_DEGREE (4)
# define index_north  index_north_300x4
# define index_south  index_south_300x4
# define index_east   index_east_300x4
# define index_west   index_west_300x4
# define pack_index   pack_index_300x4
# define unpack_index unpack_index_300x4
# define latlon2index latlon2index_300x4
# define index2latlon index2latlon_300x4
# define index_filename index_filename_300x4
# define load_tile    load_tile_300x4
# define fetch_tile   fetch_tile_300x4

#endif /* HIRESMAP */

#ifdef __cplusplus
extern "C" {
#endif

	struct map_coords {
		int ilon, ilat;
		int x, y;
	};

	struct map_index {
		int index;
		double x, y;
	};

	int
	index_north(int index);

	int
	index_south(int index);

	int
	index_east(int index);

	int
	index_west(int index);

	struct map_coords
	unpack_index(int index);

	int
	pack_index(int ilat, int ilon, int x, int y);

	struct map_index
	latlon2index(double lat, double lon);

	void
	index2latlon(int index, double *plat, double *plon);

	void
	index_filename(const tchar_t *dir, int index, tchar_t *filename, size_t fnlen);

	const short *
	fetch_tile(const tchar_t *dir, int index, int *pw, int *ph);

/* data no good: no good terrain elevation on this tile */
#define TILE_ERROR (-32768)

/* tile is ocean */
#define TILE_OCEAN (-32767)

#ifdef __cplusplus
}
#endif

#endif /* AV_MAP_H */

/*
 * Overrides for Emacs so that we follow Linux's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the
 * end of the file.
 * ----------------------------------------------------------------------
 * Local Variables:
 * c-file-style: "linux"
 * End:
 */
