/**
 * http://www.wayforward.net/avGPS/libav/av_mem_win.c
 * Copyright(c) 2005, 2008 Wayforward LLC
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include "av_mem.h"
#include "av_err.h"

static const tchar_t CREATEFILE_FN[] = T("CreateFile");
static const tchar_t GETFILESIZE_FN[] = T("GetFileSize");
static const tchar_t CREATEFILEMAPPING_FN[] = T("CreateFileMapping");
static const tchar_t MAPVIEWOFFILE_FN[] = T("MapViewOfFile");

int
map_file(const tchar_t *filename, struct mapped_file *pmf) {
	HANDLE hFile;

	hFile = CreateFile(filename,        /* lpFileName            */
			   GENERIC_READ,    /* dwDesiredAccess       */
			   FILE_SHARE_READ, /* dwShareMode           */
			   NULL,            /* lpSecurityAttributes  */
			   OPEN_EXISTING,   /* dwCreationDisposition */
			   FILE_ATTRIBUTE_NORMAL, /* dwFlagsAndAttrs */
			   NULL);           /* hTemplateFile         */

	if (hFile == INVALID_HANDLE_VALUE) {
		err_push_last_error(CREATEFILE_FN, filename);
		return 0;
	}

	pmf->len = GetFileSize(hFile, NULL);

	if (pmf->len == INVALID_FILE_SIZE) {
		err_push_last_error(GETFILESIZE_FN, filename);
	}
	else {
		HANDLE hMapping;

		hMapping = CreateFileMapping(hFile,
					     NULL,
					     PAGE_READONLY,
					     0, 0,
					     NULL);

		if (hMapping == NULL) {
			err_push_last_error(CREATEFILEMAPPING_FN, filename);
		}
		else {
			LPVOID pv;

			pv = MapViewOfFile(hMapping,
					   FILE_MAP_READ,
					   0, 0,
					   0);

			if (pv == NULL) {
				err_push_last_error(MAPVIEWOFFILE_FN, filename);
			}
			else {
				pmf->pv = (const void *) pv;
				pmf->hFile = hFile;
				pmf->hMapping = hMapping;

				return 1;
			}

			CloseHandle(hMapping);
		}
	}

	CloseHandle(hFile);
	return 0;
}

void
unmap_file(struct mapped_file *pmf) {
	(void) UnmapViewOfFile(pmf->pv);
	(void) CloseHandle(pmf->hMapping);
	(void) CloseHandle(pmf->hFile);
}

/*
 * Overrides for Emacs so that we follow Linux's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the
 * end of the file.
 * ----------------------------------------------------------------------
 * Local Variables:
 * c-file-style: "linux"
 * End:
 */
