/**
 * http://www.wayforward.net/avGPS/libav/av_time.c
 * Copyright(c) 2005, 2008 Wayforward LLC
 * ---------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
#ifdef HAVE_SYS_TIME_H
# include <sys/time.h>
#endif

#include <string.h>
#include <time.h>

#include "av_time.h"

/* Midnight, Jan 1, 2000 in local epoch */
static time_t unix_epoch = 946720800;

void
av_now(struct av_datetime *pdt) {

	time_t t = time(NULL);
	struct tm *ptm = gmtime(&t);

	pdt->date.year = ptm->tm_year + 1900;
	pdt->date.month = ptm->tm_mon + 1;
	pdt->date.day = ptm->tm_mday;
	pdt->time.hour = ptm->tm_hour;
	pdt->time.min = ptm->tm_min;
	pdt->time.sec = ptm->tm_sec;
	pdt->time.usecs = 0;
}

av_usecs
av_datetime_to_usecs(struct av_datetime dt) {
	struct tm tm;

	memset(&tm, 0, sizeof(tm));
	tm.tm_year = dt.date.year - 1900;
	tm.tm_mon = dt.date.month - 1;
	tm.tm_mday = dt.date.day;
	tm.tm_hour = dt.time.hour;
	tm.tm_min = dt.time.min;
	tm.tm_sec = dt.time.sec;

	return 1000000 * (av_usecs) (mktime(&tm) - unix_epoch) + dt.time.usecs;
}

struct av_datetime
av_usecs_to_datetime(av_usecs u) {
	time_t clock = (u / 1000000) + unix_epoch;
	struct tm *ptm = gmtime(&clock);

	struct av_datetime dt;

	dt.date.year = ptm->tm_year + 1900;
	dt.date.month = ptm->tm_mon + 1;
	dt.date.day = ptm->tm_mday;
	dt.time.hour = ptm->tm_hour;
	dt.time.min = ptm->tm_min;
	dt.time.sec = ptm->tm_sec;
	dt.time.usecs = u % 1000000;

	return dt;
}

#define USECS_PER_YEAR (1000000.0 * 86400.0 * 365.2422)

double
av_julian(struct av_datetime dt) {
	return 2000.0 + av_datetime_to_usecs(dt) / USECS_PER_YEAR;
}

/*
 * Overrides for Emacs so that we follow Linux's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the
 * end of the file.
 * ----------------------------------------------------------------------
 * Local Variables:
 * c-file-style: "linux"
 * End:
 */
